<?php /*
Plugin Name: kStats Reloaded
Plugin URI: http://mark.watero.us/kstats-reloaded/
Description: Based on the wordpress plugin 'Statpress Reloaded'.
Version: 0.1.0
Author: Mark Waterous
Author URI: http://mark.watero.us/
*/

global $wpdb;

define( 'KSTATS_VERSION', '0.1.0' ); // We're early in beta. I mean early.

define( 'KSTATS_TABLE_RAW',       $wpdb->prefix . 'kstats_raw' );
define( 'KSTATS_TABLE_TOTALS',    $wpdb->prefix . 'kstats_totals' );
define( 'KSTATS_TABLE_SEARCHES',  $wpdb->prefix . 'kstats_searches' );
define( 'KSTATS_TABLE_REFERRERS', $wpdb->prefix . 'kstats_referrers' );
define( 'KSTATS_TABLE_AGENTS',    $wpdb->prefix . 'kstats_agents' );
define( 'KSTATS_TABLE_SPIDERS',   $wpdb->prefix . 'kstats_spiders' );

define( 'KSTATS_PLUGIN_BASE', WP_PLUGIN_DIR . '/kstats/' );
define( 'KSTATS_PLUGIN_URL', WP_PLUGIN_URL . '/kstats/' );
define( 'KSTATS_DEF_BASE', KSTATS_PLUGIN_BASE . 'definitions/' );
define( 'KSTATS_LIB_BASE', KSTATS_PLUGIN_BASE . 'lib/' );

define( 'KSTATS_FILE_IGNORED_IPS',    KSTATS_DEF_BASE . 'ignore.dat' );
define( 'KSTATS_FILE_SPIDERS',        KSTATS_DEF_BASE . 'spiders.dat' );
define( 'KSTATS_FILE_OS',             KSTATS_DEF_BASE . 'os.dat' );
define( 'KSTATS_FILE_BROWSERS',       KSTATS_DEF_BASE . 'browsers.dat' );
define( 'KSTATS_FILE_SEARCH_QUERIES', KSTATS_DEF_BASE . 'searchqueries.dat' );



if ( is_admin() ) // We only need this on the dashboard, remove for use with other plugins like Viper007Bonds Admin Bar
	require_once( KSTATS_LIB_BASE . 'views.php' ); 

require_once( KSTATS_LIB_BASE . 'functions.php' ); // Make things tick
require_once( KSTATS_LIB_BASE . 'datetime.php' ); // Helper functions
require_once( KSTATS_LIB_BASE . 'cleanup.php' ); // Clean it up

// Only uncomment this if you're converting from StatPress Reloaded (tested with v1.5.21)
// Once run you can delete this line, or remove it if you don't plan on using it.
//if ( isset( $_GET['convert_kstats'] ) )
//	require_once ( KSTATS_LIB_BASE . 'convert.php' );


register_activation_hook( __FILE__, 'kstats_activate' ); 
register_deactivation_hook( __FILE__, 'kstats_deactivate' );

add_action( 'kstats_aggregate_hook', 'kstats_cleanup' ); // Run nightly aggregate
add_action( 'send_headers', 'kstats_collector' ); // Collect them hits

//add_action( 'pre_get_posts', 'kstats_cleanup', 10, 0 );

/**
 * This is our workhorse, if all goes well it will record as much information about
 * the current visitor as it can. Using headers and other information collection by
 * PHP with the super global $_SERVER, we'll try and determine if this is a person,
 * bot, search engine hit, and so on. All data is stored directly in the raw table
 * to be interpreted later by the aggregate function and statistics view.
 *
 * @version 0.1.0
 * @Category Main
 * @return void
 */
 
function kstats_collector() {
	global $wpdb, $current_user;

	// Don't record statistics for the blog owner(s)
	if ( current_user_can( 'create_users' ) )
		return;

	$arr = array();

	// Check the IP against our ignore list, exit if matched
	if ( kstats_ignore_ip ( $_SERVER['REMOTE_ADDR'] ) )
		return;


	$arr['url'] = kstats_get_url(); // Request URI

	// Don't collect stats for these areas of the site
	if ( preg_match( '~(robots\.txt|wp-content/(mu-)?(plugins|themes)|wp-admin)~', $arr['url'] ) )
		return;

	// File extensions that we want to ignore
	$arr['ext'] = pathinfo( $arr['url'], PATHINFO_EXTENSION );
	if ( ! empty( $arr['ext'] ) && preg_match( '~ico|css|js~', $arr['ext'] ) )
		return;


	// Set variables
	$arr['referrer']   = ( isset( $_SERVER['HTTP_REFERER'] ) ) ? htmlentities( $_SERVER['HTTP_REFERER'], ENT_COMPAT, 'utf-8' ) : '';
	$arr['user_agent'] = ( isset( $_SERVER['HTTP_USER_AGENT'] ) ) ? htmlentities( $_SERVER['HTTP_USER_AGENT'], ENT_COMPAT, 'utf-8' ) : '';

	$arr['spider'] = kstats_trap_spider( $arr['user_agent'] );
          
	// Default
	$arr['feed'] = $arr['os'] = $arr['browser'] = $arr['search_engine'] = $arr['search_terms'] = $arr['feed'] = FALSE;

	// Not a spider? Who is it? TELL ME.
	if ( $arr['spider'] === FALSE ) {
		$arr['feed']    = kstats_trap_feed( $arr['url'] );
        $arr['os']      = kstats_trap_os( $arr['user_agent'] );
		$arr['browser'] = kstats_trap_browser( $arr['user_agent'] );
		list( $arr['search_engine'], $arr['search_terms'] ) = explode( '|', kstats_trap_search_query( $arr['referrer'] ) );
	}

	$query  = sprintf( "INSERT INTO `%s` ( timestamp, ip, url, user_agent, referrer, spider, search_engine, search_terms, os, browser, feed, user ) ", KSTATS_TABLE_RAW );
	$query .= sprintf( "VALUES ( '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')",
								gmdate( 'Y-m-d H:i:s', kstats_current_time() ),
								$_SERVER['REMOTE_ADDR'],
								mysql_real_escape_string( $arr['url'] ),
								mysql_real_escape_string( $arr['user_agent'] ),
								mysql_real_escape_string( $arr['referrer'] ),
								$arr['spider'],
								$arr['search_engine'],
								$arr['search_terms'],
								$arr['os'],
								$arr['browser'],
								$arr['feed'],
								$current_user->user_login );

	$wpdb->query( $query ); // RUN IT!

}


/**
 * The activation process, schedules the nightly cron event and creates the database
 * tables we'll be using to store our data. All tables are MyIsam for speed with the
 * single exception of InnoDB on our raw data table, as this should be the only table
 * we need to make use of row level locking on.
 *
 * @ver 0.1.0
 * @category Main
 * @return void
 */

function kstats_activate() {
	global $wpdb;

	$first_run = strtotime( date( 'Y/m/d', strtotime( '+1 day', kstats_current_time() ) ) . ' 12:30am' );

	wp_schedule_event( $first_run, 'daily', 'kstats_aggregate_hook' );

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

	$tables = array( 'raw'       => $wpdb->prefix . 'kstats_raw',
					 'totals'    => $wpdb->prefix . 'kstats_totals',
					 'searches'  => $wpdb->prefix . 'kstats_searches',
					 'referrers' => $wpdb->prefix . 'kstats_referrers',
					 'agents'    => $wpdb->prefix . 'kstats_agents',
					 'spiders'   => $wpdb->prefix . 'kstats_spiders' );

	$tables['raw'] = "CREATE TABLE IF NOT EXISTS `" . $tables['raw'] . "` (
						`id` int UNSIGNED NOT NULL AUTO_INCREMENT,
						`timestamp` datetime NOT NULL,
						`ip` tinytext CHARACTER SET latin1,
						`url` text CHARACTER SET latin1,
						`user_agent` text CHARACTER SET latin1,
						`referrer` text CHARACTER SET latin1,
						`spider` tinytext CHARACTER SET latin1,
						`search_engine` tinytext CHARACTER SET latin1,
						`search_terms` text CHARACTER SET latin1,
						`os` tinytext CHARACTER SET latin1,
						`browser` tinytext CHARACTER SET latin1,
						`feed` tinytext CHARACTER SET latin1,
						`user` tinytext CHARACTER SET latin1,
						`preserved` tinyint(1) DEFAULT NULL,
						UNIQUE KEY `id` (`id`),
						KEY `timestamp` (`timestamp`)
					  ) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_bin ";

	$tables['agents'] = "CREATE TABLE IF NOT EXISTS `" . $tables['agents'] . "` (
						   `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
						   `user_agent` text COLLATE utf8_bin NOT NULL,
						   `name` varchar(255) COLLATE utf8_bin NOT NULL,
						   `count` int(10) unsigned NOT NULL,
						   PRIMARY KEY (`ID`)
						  ) ENGINE=MyIsam  DEFAULT CHARSET=utf8 COLLATE=utf8_bin ";

	$tables['referrers'] = "CREATE TABLE IF NOT EXISTS `" . $tables['referrers'] . "` (
							  `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
							  `viewed` text COLLATE utf8_bin NOT NULL,
							  `referrer` text COLLATE utf8_bin NOT NULL,
							  `count` int(10) unsigned NOT NULL,
							  PRIMARY KEY (`ID`)
							) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_bin ";

	$tables['searches'] = "CREATE TABLE IF NOT EXISTS `" . $tables['searches'] . "` (
							 `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
							 `terms` varchar(255) COLLATE utf8_bin NOT NULL,
							 `engine` varchar(255) COLLATE utf8_bin NOT NULL,
							 `viewed` text COLLATE utf8_bin NOT NULL,
							 `referrer` text COLLATE utf8_bin NOT NULL,
							 `count` int(10) unsigned NOT NULL,
							 PRIMARY KEY (`ID`),
							 KEY `terms` (`terms`)
						   ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_bin ";

	$tables['spiders'] = "CREATE TABLE IF NOT EXISTS `" . $tables['spiders'] . "` (
							`ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
							`user_agent` text COLLATE utf8_bin NOT NULL,
							`spider` varchar(255) COLLATE utf8_bin NOT NULL,
							`url` text COLLATE utf8_bin NOT NULL,
							`count` int(10) unsigned NOT NULL,
							PRIMARY KEY (`ID`)
						  ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_bin ";

	$tables['totals'] = "CREATE TABLE IF NOT EXISTS `" . $tables['totals'] . "` (
						   `type` enum('charts','totals') COLLATE utf8_bin NOT NULL DEFAULT 'charts',
						   `name` varchar(12) COLLATE utf8_bin NOT NULL,
						   `total` int(10) unsigned NOT NULL,
						   `last` int(10) unsigned NOT NULL,
						   `this` int(10) unsigned NOT NULL,
						   `yesterday` int(10) unsigned NOT NULL,
						   PRIMARY KEY (`name`)
						 ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_bin ";

	foreach ( $tables as $table )
		dbDelta( $table );

}


/**
 * Deactivation process, unschedule our wp-cron events, and drop the database tables
 *
 * @ver 0.1.0
 * @category Main
 * @return void
 */

function kstats_deactivate() {
	global $wpdb;

	wp_clear_scheduled_hook ( 'kstats_aggregate_hook' );

	$tables = array( 'raw'       => $wpdb->prefix . 'kstats_raw',
					 'totals'    => $wpdb->prefix . 'kstats_totals',
					 'searches'  => $wpdb->prefix . 'kstats_searches',
					 'referrers' => $wpdb->prefix . 'kstats_referrers',
					 'agents'    => $wpdb->prefix . 'kstats_agents',
					 'spiders'   => $wpdb->prefix . 'kstats_spiders' );

	$wpdb->query( "DROP TABLE IF EXISTS `" . $tables['raw'] . "`" );
	$wpdb->query( "DROP TABLE IF EXISTS `" . $tables['agents'] . "`" );
	$wpdb->query( "DROP TABLE IF EXISTS `" . $tables['referrers'] . "`" );
	$wpdb->query( "DROP TABLE IF EXISTS `" . $tables['searches'] . "`" );
	$wpdb->query( "DROP TABLE IF EXISTS `" . $tables['spiders'] . "`" );
	$wpdb->query( "DROP TABLE IF EXISTS `" . $tables['totals'] . "`" );

}
