<?php

/**
 * This function runs the actual cleanup process every night when the scheduled event
 * is triggered.
 *
 * @ver 0.1.0
 * @category Cleanup
 * @return void
 */

function kstats_cleanup() {
	global $wpdb, $kstatstime;

	$kstatstime->start_timer();

	$query[] = sprintf( "SELECT SUBSTR( timestamp, 1, 10 ) AS timestamp, COUNT( DISTINCT ip ) AS visitors, COUNT( timestamp ) AS pageviews FROM `%s` WHERE feed = '' AND spider = '' AND preserved IS NULL GROUP BY SUBSTR( timestamp, 1, 10 ) WITH ROLLUP", KSTATS_TABLE_RAW );
	$one = $wpdb->get_results( end( $query ), ARRAY_A );

	$query[] = sprintf( "SELECT SUBSTR( timestamp, 1, 10 ) AS timestamp, COUNT( ip ) AS spiders FROM `%s` WHERE spider <> '' AND preserved IS NULL GROUP BY SUBSTR( timestamp, 1, 10 ) WITH ROLLUP", KSTATS_TABLE_RAW );
	$two = $wpdb->get_results( end( $query ), ARRAY_A );
	
	$query[] = sprintf( "SELECT SUBSTR( timestamp, 1, 10 ) AS timestamp, COUNT( ip ) AS feeds FROM `%s` WHERE feed <> '' AND preserved IS NULL GROUP BY SUBSTR( timestamp, 1, 10 ) WITH ROLLUP", KSTATS_TABLE_RAW );
	$three = $wpdb->get_results( end( $query ), ARRAY_A );

	$rows = count( $one ) - 1;

	for ( $i = 0; $i <= $rows; $i++ ) {

		$timestamp = isset( $one[$i]['timestamp'] )   ? $one[$i]['timestamp'] :
				   ( isset( $two[$i]['timestamp'] )   ? $two[$i]['timestamp'] :
				   ( isset( $three[$i]['timestamp'] ) ? $three[$i]['timestamp'] : 'totals' ) );

		if ( $timestamp == $kstatstime->today )
			$idx_today = $i;
		else if ( $timestamp == $kstatstime->yesterday )
			$idx_yesterday = $i;

		$data['days'][$i]['timestamp'] = $timestamp;
		$data['days'][$i]['visitors']  = isset( $one[$i]['visitors'] ) ? $one[$i]['visitors'] : 0;
		$data['days'][$i]['pageviews'] = isset( $one[$i]['pageviews'] ) ? $one[$i]['pageviews'] : 0;
		$data['days'][$i]['spiders']   = isset( $two[$i]['spiders'] ) ? $two[$i]['spiders'] : 0;
		$data['days'][$i]['feeds']     = isset( $three[$i]['feeds'] ) ? $three[$i]['feeds'] : 0;

		if ( $timestamp == $kstatstime->last_month_day )
			$changed_month = $i;
		else if ( ! isset( $changed_month ) )
			$changed_month = 0;
	}

	$tmp_totals = array_pop( $data['days'] );

	$data['totals']['visitors']['total']  = $tmp_totals['visitors'];
	$data['totals']['pageviews']['total'] = $tmp_totals['pageviews'];
	$data['totals']['spiders']['total']   = $tmp_totals['spiders'];
	$data['totals']['feeds']['total']     = $tmp_totals['feeds'];

	unset( $tmp_totals );

	if ( $changed_month ) {

		$data['this_month'] = array_slice( $data['days'], $changed_month + 1 );
		$data['last_month'] = array_slice( $data['days'], 0, $changed_month + 1 );

		foreach ( $data['last_month'] as $row ) {
			$data['totals']['visitors']['last_month']  += $row['visitors'];
			$data['totals']['pageviews']['last_month'] += $row['pageviews'];
			$data['totals']['spiders']['last_month']   += $row['spiders'];
			$data['totals']['feeds']['last_month']     += $row['feeds'];
		}

		foreach ( $data['this_month'] as $row ) {
			$data['totals']['visitors']['this_month']  += $row['visitors'];
			$data['totals']['pageviews']['this_month'] += $row['pageviews'];
			$data['totals']['spiders']['this_month']   += $row['spiders'];
			$data['totals']['feeds']['this_month']     += $row['feeds'];
		}

		unset( $data['this_month'] );
		unset( $data['last_month'] );

	}
	else {

		$data['totals']['visitors']['last_month']  = 0;
		$data['totals']['pageviews']['last_month'] = 0;
		$data['totals']['spiders']['last_month']   = 0;
		$data['totals']['feeds']['last_month']     = 0;

		foreach ( $data['days'] as $row ) {
			$data['totals']['visitors']['this_month']  += $row['visitors'];
			$data['totals']['pageviews']['this_month'] += $row['pageviews'];
			$data['totals']['spiders']['this_month']   += $row['spiders'];
			$data['totals']['feeds']['this_month']     += $row['feeds'];
		}

	}

	// This should never run on the same day twice, but what if somebody changes their timezone dramatically? Skewed results = bad
	$query[] = sprintf( "SELECT name AS timestamp, total AS visitors, last AS pageviews, this AS spiders, yesterday AS feeds FROM `%s` WHERE name >= '%s' AND type = 'charts'", KSTATS_TABLE_TOTALS, $kstatstime->yesterday );
	$results = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( is_array( $results ) ) {
		$res_yesterday = $results[0];
		$res_today = isset( $results[1] ) ? $results[1] : NULL;
	}

			
	if ( isset( $res_yesterday ) ) {
		$data['totals']['visitors']['yesterday']  = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['visitors']  + $res_yesterday['visitors']  : $res_yesterday['visitors'];
		$data['totals']['pageviews']['yesterday'] = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['pageviews'] + $res_yesterday['pageviews'] : $res_yesterday['pageviews'];
		$data['totals']['spiders']['yesterday']   = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['spiders']   + $res_yesterday['spiders']   : $res_yesterday['spiders'];
		$data['totals']['feeds']['yesterday']     = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['feeds']     + $res_yesterday['feeds']     : $res_yesterday['feeds'];
	}
	else {
		$data['totals']['visitors']['yesterday']  = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['visitors'] : 0;
		$data['totals']['pageviews']['yesterday'] = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['pageviews'] : 0;
		$data['totals']['spiders']['yesterday']   = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['spiders'] : 0;
		$data['totals']['feeds']['yesterday']     = isset( $idx_yesterday ) ? $data['days'][$idx_yesterday]['feeds'] : 0;
	}

	// Skewed results = bad.... this can wind up counting the same unique hit as two...
	// Might have to take that chance. This portion should almost never if ever run, and we don't have the old data to run against anyways.
	if ( isset( $idx_today ) && isset( $res_today ) ) {
		$data['days'][$idx_today]['visitors']  += $res_today['visitors'];
		$data['days'][$idx_today]['pageviews'] += $res_today['pageviews'];
		$data['days'][$idx_today]['spiders']   += $res_today['spiders'];
		$data['days'][$idx_today]['feeds']     += $res_today['feeds'];
	}


	// Let's pretend somebody accesses the blog after the scheduled event runs...
	// Then doesn't for another 3 weeks before it gets triggered again...
	// (it could happen)
	$number_of_days = count( $data['days'] );
	if ( $number_of_days > 21)
		$data['days'] = array_splice( $data['days'], -21 );

	// Delete everything older than 21 days ago
	$query[] = sprintf( "DELETE FROM `%s` WHERE type = 'charts' AND name < '%s'", KSTATS_TABLE_TOTALS, $kstatstime->last_21[21] );
	$wpdb->query( end( $query ) );

	// Update totals
	$query[] = sprintf( "SELECT name, total, last AS last_month, this AS this_month, yesterday FROM `%s` WHERE `type` = 'totals'", KSTATS_TABLE_TOTALS );
	$result = $wpdb->get_results( end( $query ), ARRAY_A );

	foreach ( $result as $row ) {
		
		$data['totals'][$row['name']]['total'] += $row['total'];

		if ( $changed_month )
			$data['totals'][$row['name']]['last_month'] += $row['this_month'];
		else {
			$data['totals'][$row['name']]['last_month'] += $row['last_month'];
			$data['totals'][$row['name']]['this_month'] += $row['this_month'];
		}

	}

	$build_query = array();

	foreach ( $data['days'] as $day )
		$build_query[] = sprintf( "('charts', '%s', '%d', '%d', '%d', '%d')", $day['timestamp'], $day['visitors'], $day['pageviews'], $day['spiders'], $day['feeds'] );
	
	foreach ( $data['totals'] as $name => $total )
		$build_query[] = sprintf( "('totals', '%s', '%d', '%d', '%d', '%d')", $name, $total['total'], $total['last_month'], $total['this_month'], $total['yesterday'] );

	$query[] = sprintf( "REPLACE INTO `%s` VALUES %s", KSTATS_TABLE_TOTALS, implode( ', ', $build_query ) );
	$wpdb->query ( end( $query ) );

/*	kstats_cleanup_referrers();
	kstats_cleanup_searches();
	kstats_cleanup_agents();
	kstats_cleanup_spiders();
*/
	$query[] = sprintf( "SELECT COUNT( timestamp ) AS total FROM `%s`", KSTATS_TABLE_RAW );
	$limit = $wpdb->get_row( end( $query ) )->total - 30;

	$query[] = sprintf( "DELETE FROM `%s` ORDER BY timestamp LIMIT %d", KSTATS_TABLE_RAW, $limit );
	$wpdb->query( end( $query ) );

	$query[] = sprintf( "UPDATE `%s` SET preserved = 1", KSTATS_TABLE_RAW );
	$wpdb->query( end( $query ) );

/*
	$query[] = sprintf( "CREATE TEMPORARY TABLE `%skstats_old_data` SELECT * FROM `%s` ORDER BY timestamp DESC LIMIT 30", $wpdb->prefix, KSTATS_TABLE_RAW );
	$wpdb->query( end( $query ) );

	$wpdb->query( "DELETE FROM `" . KSTATS_TABLE_RAW . "` WHERE SUBSTR( timestamp, 1, 10 ) <> '" . $kstatstime->today . "'" );
	$wpdb->query( "INSERT INTO `" . KSTATS_TABLE_RAW . "` SELECT * FROM `old_data`" );
	$wpdb->query( "UPDATE `" . KSTATS_TABLE_RAW . "` SET preserved = 1 WHERE SUBSTR( timestamp, 1, 10) <> '" . $kstatstime->today . "'" ); */

	update_option( 'kstats_last_run', $kstatstime->today );
	//$kstatstime->end_timer(); die; last test took 0.07 seconds.
}


/**
 * Grabs data from the raw table and adds it to our aggregate table
 * for use later.
 *
 * @ver 0.1.0
 * @category Cleanup
 * @param string $table This is only for the conversion program and probably won't ever be used outside of that
 * @return void
 */

function kstats_cleanup_searches ( $table = KSTATS_TABLE_RAW ) {
	global $wpdb;

	$results = array();

	$query = sprintf( "SELECT search_terms AS terms, search_engine AS engine, url AS viewed, referrer FROM `%s` WHERE search_terms <> '' AND preserved IS NULL ORDER BY id DESC", $table );
	$results['raw'] = $wpdb->get_results( $query, ARRAY_A );

	if ( ! is_array( $results['raw'] ) )
		return;

	$query = sprintf( "SELECT terms, engine, viewed, referrer, count FROM `%s` ORDER BY count DESC", KSTATS_TABLE_SEARCHES );
	$results['current'] = $wpdb->get_results( $query, ARRAY_A );

	if ( ! is_array( $results['current'] ) )
		$results['current'] = array();

	$results = array_merge( $results['raw'], $results['current'] );

	$build = array();
	$mdsums = array();

	foreach ( $results as $the ) {
		$md = md5( $the['terms'] . $the['engine'] . $the['viewed'] );
		
		if ( isset( $mdsums[$md] ) && isset( $the['count'] ) )
			$mdsums[$md]['count'] = $mdsums[$md]['count'] + $the['count'];
		else if ( isset( $mdsums[$md] ) )
			$mdsums[$md]['count']++;
		else
			$mdsums[$md]['count'] = 1;

		$mdsums[$md]['data'] = implode( '||', $the );
	}

	arsort( $mdsums );

	$query = sprintf( "TRUNCATE TABLE `%s`", KSTATS_TABLE_SEARCHES );
	$wpdb->query( $query );

	foreach ( $mdsums as $the ) {
		$parts = explode( '||', $the['data'] );
		$query_parts[] = sprintf ( "('%s', '%s', '%s', '%s', '%d')", $parts[0], $parts[1], $parts[2], $parts[3], $the['count'] );
	}

	kstats_cleanup_run_query( KSTATS_TABLE_SEARCHES, array ( 'terms', 'engine', 'viewed', 'referrer', 'count' ), $query_parts );
	return TRUE;
}


/**
 * Builds the referrer aggregate table. Excludes local referrers to display
 * an accurate representation of the top 20 most active referrer urls. Maintains
 * data from referrers who don't reach the top 20 to guarantee that if any particular
 * referrer overtakes another, the data remains accurate at all times.
 *
 * @ver 0.1.0
 * @category Cleanup
 * @param string $table This is only for the conversion program and probably won't ever be used outside of that
 * @return void
 */

function kstats_cleanup_referrers ( $table = KSTATS_TABLE_RAW ) {
	global $wpdb;

	$query = sprintf( "SELECT url AS viewed, referrer FROM `%s` WHERE ( referrer NOT LIKE '%s%%' AND referrer <> '' AND search_engine = '' AND url <> '' AND preserved IS NULL )", $table, get_bloginfo( 'url' ) );
//	$query = sprintf( "SELECT url AS viewed, referrer FROM `%s` WHERE ( referrer NOT LIKE '%s%%' AND referrer <> '' AND search_engine = '' AND url <> '' AND preserved IS NULL )", $table, 'http://mark.watero.us' ); // For testing using stats from my blog
	$results['raw'] = $wpdb->get_results( $query, ARRAY_A );

	if ( ! is_array( $results['raw'] ) )
		return;

	$query = sprintf( "SELECT viewed, referrer, count FROM `%s` ORDER BY count DESC", KSTATS_TABLE_REFERRERS );
	$results['current'] = $wpdb->get_results( $query, ARRAY_A );

	if ( ! is_array( $results['current'] ) )
		$results['current'] = array();

	$results = array_merge( $results['raw'], $results['current'] );

	$build = array();
	$mdsums = array();

	foreach ( $results as $the ) {
		$md = md5( $the['viewed'] . $the['referrer'] );
		
		if ( isset( $mdsums[$md] ) && isset( $the['count'] ) )
			$mdsums[$md]['count'] = $mdsums[$md]['count'] + $the['count'];
		else if ( isset( $mdsums[$md] ) )
			$mdsums[$md]['count']++;
		else
			$mdsums[$md]['count'] = 1;

		$mdsums[$md]['data'] = implode( '||', $the );
	}

	arsort( $mdsums );

	$query = sprintf( "TRUNCATE TABLE `%s`", KSTATS_TABLE_REFERRERS );
	$wpdb->query( $query );

	foreach ( $mdsums as $the ) {
		$parts = explode( '||', $the['data'] );
		$query_parts[] = sprintf( "('%s', '%s', '%d')", $parts[0], $parts[1], $the['count'] );
	}

	kstats_cleanup_run_query( KSTATS_TABLE_REFERRERS, array( 'viewed', 'referrer', 'count' ), $query_parts );
	return TRUE;
}


/**
 * User Agents aggregate information including users operating system and browser
 * where available.
 *
 * @ver 0.1.0
 * @category Cleanup
 * @param string $table This is only for the conversion program and probably won't ever be used outside of that
 * @return void
 */

function kstats_cleanup_agents ( $table = KSTATS_TABLE_RAW ) {
	global $wpdb;

	$query = sprintf( "SELECT user_agent, CONCAT_WS(' ', os, browser) AS name FROM `%s` WHERE user_agent <> '' AND CONCAT(os, browser) <> '' AND preserved IS NULL GROUP BY ip ORDER BY id DESC", $table );
	$results['raw'] = $wpdb->get_results( $query, ARRAY_A );

	if ( ! is_array( $results['raw'] ) )
		return;

	$query = sprintf( "SELECT user_agent, name, count FROM `%s` ORDER BY count DESC", KSTATS_TABLE_AGENTS );
	$results['current'] = $wpdb->get_results( $query, ARRAY_A );

	if ( ! is_array( $results['current'] ) )
		$results['current'] = array();

	$results = array_merge( $results['raw'], $results['current'] );

	$build = array();
	$mdsums = array();

	foreach ( $results as $the ) {
		$md = md5( $the['user_agent'] . $the['name'] );
		
		if ( isset( $mdsums[$md] ) && isset( $the['count'] ) )
			$mdsums[$md]['count'] = $mdsums[$md]['count'] + $the['count'];
		else if ( isset( $mdsums[$md] ) )
			$mdsums[$md]['count']++;
		else
			$mdsums[$md]['count'] = 1;

		$mdsums[$md]['data'] = implode( '||', $the );
	}

	arsort( $mdsums );

	$query = sprintf( "TRUNCATE TABLE `%s`", KSTATS_TABLE_AGENTS );
	$wpdb->query( $query );

	foreach ( $mdsums as $the ) {
		$parts = explode( '||', $the['data'] );
		$query_parts[] = sprintf( "('%s', '%s', '%d')", mysql_real_escape_string( $parts[0] ), $parts[1], $the['count'] );
	}

	kstats_cleanup_run_query( KSTATS_TABLE_AGENTS, array( 'user_agent', 'name', 'count' ), $query_parts );
	return TRUE;
}


/**
 * Aggregates data on crawler visits, including differentiation
 *
 * @ver 0.1.0
 * @category Cleanup
 * @param string $table This is only for the conversion program and probably won't ever be used outside of that
 * @return void
 */

function kstats_cleanup_spiders ( $table = KSTATS_TABLE_RAW ) {
	global $wpdb;

	$query = array();

	$query[] = sprintf( "SELECT user_agent, spider, url, UNIX_TIMESTAMP(timestamp) AS timestamp FROM `%s` WHERE spider <> '' AND url <> '' AND preserved IS NULL ORDER BY id DESC", $table );
	$results['raw'] = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( ! is_array ( $results['raw'] ) )
		return;

	$query[] = sprintf( "SELECT user_agent, name, count FROM `%s` ORDER BY count DESC", KSTATS_TABLE_SPIDERS );
	$results['current'] = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( ! is_array( $results['current'] ) )
		$results['current'] = array();

	$results = array_merge( $results['raw'], $results['current'] );

	$build = array();
	$mdsums = array();

	foreach ( $results as $the ) {
		$md = md5( $the['user_agent'] . $the['spider'] );

		if ( isset( $mdsums[$md] ) && isset( $the['count'] ) )
			$mdsums[$md]['count'] = $mdsums[$md]['count'] + $the['count'];
		else if ( isset( $mdsums[$md] ) )
			$mdsums[$md]['count']++;
		else
			$mdsums[$md]['count'] = 1;

		if ( $the['url'] != '/robots.txt' ) {
			if ( isset( $mdsums[$md]['url'] ) && $the['timestamp'] > $mdsums[$md]['urlts'] ) {
				$mdsums[$md]['url'] = $the['url'];
				$mdsums[$md]['urlts'] = $the['timestamp'];
			}
			else if ( ! isset( $mdsums[$md]['url'] ) ) {
				$mdsums[$md]['url'] = $the['url'];
				$mdsums[$md]['urlts'] = $the['timestamp'];
			}
		}
			
		$mdsums[$md]['data'] = implode( '||', $the );
	}

	arsort( $mdsums );

	$query[] = sprintf( "TRUNCATE TABLE `%s`", KSTATS_TABLE_SPIDERS );
	$wpdb->query( end( $query ) );

	foreach ( $mdsums as $the ) {
		if ( empty( $the['url'] ) )
			continue;

		$parts = explode( '||', $the['data'] );
		$query_parts[] = sprintf( "('%s', '%s', '%s', '%d')", mysql_real_escape_string( $parts[0] ), $parts[1], $the['url'], $the['count'] );
	}

	kstats_cleanup_run_query( KSTATS_TABLE_SPIDERS, array( 'user_agent', 'spider', 'url', 'count' ), $query_parts );
	return TRUE;
}


/**
 * Given a table name, an array of fields to use, and an array of data,
 * this helper method builds and runs the query to insert the data into our
 * aggregate tables.
 *
 * @ver 0.1.0
 * @category Cleanup
 * @param string $table  The name of the table we're running the query on
 * @param array  $fields An indexed array of fields to use in the query
 * @param array  $values An indexed array of preformatted values to insert into their respective fields
 * @param int    $per    How many sets of data to run in one query
 * @return bool True on success, false on failure
 */

function kstats_cleanup_run_query ( $table, $fields = array(), $values = array(), $per = 200 ) {
	global $wpdb;

	if ( empty( $table ) || empty( $fields ) || empty( $values ) )
		return FALSE;

	$total_rows = count( $values );

	// Are there more rows to add than are allowed in one query?
	if ( $total_rows > $per ) {
		$iterations = ceil( $total_rows / $per );

		// Chop them up into little bits
		for ( $i = 1; $i <= $iterations; $i++ ) {
			$start = ( $i - 1 ) * $per;
			$query_parts[] = array_slice( $values, $start, $per );
		}

	}
	else
		$query_parts[] = $values;

	foreach ( $query_parts as $query ) {
		$query = sprintf( "INSERT INTO `%s` (%s) VALUES %s", $table, implode( ', ', $fields ), implode( ', ', $query ) );
		$wpdb->query( $query );
	}

	return TRUE;

}
