<?php

/**
 * @ignore
 */

$kstatstime = new kstats_datetime;

/**
 * WTF knows what's going on with the time settings around here... UTC tells
 * me its 8am in the morning when I know it's 1am right now, so to hell with it,
 * let's use current_timestamp until we know better.
 * @ignore
 */

function kstats_current_time() {
	$offset = get_option( 'gmt_offset' );
	list( $month, $day, $year, $hour, $minute, $second ) = explode( ' ', gmdate( 'n j Y G i s' ) );
	return gmmktime( $hour + $offset, $minute, $second, $month, $day, $year );
}


class kstats_datetime {
	private $stopwatch;
	public $today;
	public $yesterday;
	public $this_month;
	public $last_month;
	public $last_month_day;
	public $is_new_month;
	public $last_21 = array();
	public $start_of_week;

	public function __construct() {
		$this->today          = gmdate ( 'Y-m-d', kstats_current_time() );
		$this->is_new_month   = $this->get_time_newmonth ( $this->today );
		$this->last_21        = $this->get_time_last_21_days ( $this->today );
		$this->yesterday      = $this->last_21[1];
		$this->this_month     = substr ( $this->today, 0, 7 );
		$this->last_month     = $this->get_time_lastmonth();
		$this->last_month_day = $this->get_time_lastmonth_day();
		$this->start_of_week  = get_option( 'start_of_week' );
	}


	/**
	 * Timer function, records the start time and returns it
	 * @ignore
	 */

	public function start_timer() {
		$mtime = explode( ' ', microtime() );
		$this->stopwatch = $mtime[1] + $mtime[0];
	}


	/**
	 * Timer function, based on the provided start time returns the total time elapsed as a string
	 * @ignore
	 */

	public function end_timer( $ech = TRUE, $msg = 'This page was created in %s seconds' ) {
		$mtime = explode( ' ', microtime() );
		$total = ( ( $mtime[1] + $mtime[0] ) - $this->stopwatch );

		return ( $ech ) ? printf( $msg, $total ) : sprintf( $msg, $total );
	}


	private function get_time_last_21_days ( $time ) {

		$parts = explode ( '-', $time );

		$day   = $parts[2] + 1;
		$month = $parts[1];
		$year  = $parts[0];

		$days = array();

		for ( $i = 1; $i < 23; $i++ ) {
			$day = $day - 1;

			if ( $day == 0 ) {
				$day = kstats_get_lastmonth_day ( $parts[1], $parts[0] );
			}

			$day = ( strlen ( $day ) == 1 ) ? '0' . $day : $day;
			$days[] = $year . '-' . $month . '-' . $day;
		}

		return $days;

	}


	private function get_time_newmonth ( $today ) {
		$parts = explode ( '-', $today );

		return ( (int) $parts[2] == 01 ) ? TRUE : FALSE;
	}

	private function get_time_lastmonth () {

		extract ( getdate ( kstats_current_time() ) );

		$mon--;

		if ( $mon === 0 ) {
			$year--;
			$mon = 12;
		}

		return sprintf ( $year . '-%02d', $mon);
	}


	private function get_time_lastmonth_day ( $month = NULL, $year = NULL ) {

		// Set the current month and year
	//	echo $month = ( isset ( $month ) ) ? $month : date ( 'm' ) ;
	//	echo $year  = ( isset ( $year ) ) ? $year : date ( 'Y' ) ;

		// return the last day of the requested month
		return date ( 'Y-m-d', strtotime ( '-1 second',	strtotime ( date ( 'm' ) . '/01/' . date ( 'Y' ) . ' 00:00:00' ) ) );
	}

}
