<?php

add_action( 'wp_dashboard_setup', 'kstats_add_dashboard_widget' );
add_action( 'admin_menu', 'kstats_add_pages' );
add_action( 'admin_print_styles', 'kstats_print_styles' ); 

/**
 * Activate the dashboard widget
 * @ignore
 */
 
function kstats_add_dashboard_widget() {
	wp_add_dashboard_widget( 'kstats', 'Blog Statistics', 'kstats_display_dashboard_widget' );	
} 


/**
 * Activate the dashboard page(s)
 * @ignore
 */

function kstats_add_pages() {
	$page = add_submenu_page( 'index.php', __('Statistics Overview'), __('kStats Reloaded'), 'create_users', 'kstats/overview', 'kstats_display' );
}

/**
 * Append the kstats stylesheet to the dashboard
 * @ignore
 */

function kstats_print_styles() {
	wp_enqueue_style( 'kstats-css', KSTATS_PLUGIN_URL . 'kstats.css', '', KSTATS_VERSION, 'all' );
}

/**
 * This function displays the dashboard widget, which includes an overview of your
 * statistics, as well as the dynamically generated charts.
 *
 * @ver 0.1.0
 * @category Views
 * @return void
 */

function kstats_display_dashboard_widget() {
	global $wpdb, $kstatstime; 

	$kstatstime->start_timer();

	extract( kstats_get_chart_totals() );
?>
<style type="text/css">
.kstatsfat tbody td { font-size: 11px; border-bottom: 1px dotted #e2e2e2; border-left: 1px dotted #e0e0e0; padding: 2px 0 2px 3px; }
.kstatsfat tbody td:first-child { border-left: none; padding: 2px 0; }

.boxes { width: 8px; height: 8px; float: left; margin: 2px 3px 0 0; }
</style>
<?php

	kstats_display_overview( $totals, FALSE );
	kstats_display_charts( $charts, $max, 11 );

	echo '<a href="' . $_SERVER['REQUEST_URI'] . '?page=kstats/overview" style="display: block; text-align: center; font-size: 10px; padding: 10px 0 5px;">View Statistics</a>';
	echo '<!-- ' . $kstatstime->end_timer( FALSE ) . '-->';
}




/**
 * KreativKats Statistics primary view, 'overview'
 *
 */

function kstats_display () {
	global $wpdb, $kstatstime, $current_user, $blog_id;

	extract ( kstats_get_chart_totals() );

?>

	<div class="wrap">
    <div id="icon-index" class="icon32"><br /></div>
    <h2>Blog Statistics for <?php bloginfo ( 'title' ); ?></h2>


<?php

	kstats_display_overview( $totals );
	kstats_display_charts( $charts, $max );

?>

	<h2>Recent Page Views</h2>
    <table class="kstatsfat">
    <thead>
    <tr><th scope="col">Time</th>
    	<th scope="col" style="padding-left: 27px;">IP</th>
    	<th scope="col">Page Viewed</th>
    	<th scope="col">OS</th>
    	<th scope="col">Browser</th>
	</tr>
    </thead>

	<tbody>

<?php
	$query = sprintf( "SELECT timestamp, ip, url, os, browser FROM `%s` WHERE feed = '' AND spider = '' ORDER BY timestamp DESC LIMIT 20", KSTATS_TABLE_RAW );
	$recent = $wpdb->get_results( $query, ARRAY_A );


	if ( is_null( $recent ) || ! is_array( $recent ) || empty( $recent ) ): ?>
    <tr><td colspan="5" style="padding: 4em; text-align: center;"><strong>kStats hasn't had a chance to record any hits yet.</strong><br />It could be broken. Unless you just turned it on.<br /><span style="font-size: 9px;">It's probably just broken.</span></td></tr>
    <?php
	else: foreach ( $recent as $the ): ?>
	<tr><td style="font-size: 10px; color: #aaa;"><?php echo date( 'F j, Y g:ia', strtotime ( $the['timestamp'] ) ); ?></td>
		<td><img src="http://api.hostip.info/flag.php?ip=<?php echo $the['ip']; ?>" width="18" height="12" style="margin-right: 5px; position: relative; top: 2px; border: 1px solid #eee;"><?php echo $the['ip']; ?></td>
		<td><?php echo kstats_truncate( $the['url'], 30 ); ?></td>
		<td style="font-size: 10px; color: #aaa;"><?php echo $the['os']; ?></td>
		<td style="font-size: 10px; color: #aaa;"><?php echo $the['browser']; ?></td>
	</tr> <?php
	endforeach; endif;
?>

	</tbody>
    </table>

    <div style="margin: 2em 0; width: 49%; float: left; ">
    <h2>Top Search Terms</h2>
    <table class="kstatsfat">
    <thead>
    <tr><th scope="col">Terms</th>
        <th scope="col">Engine</th>
        <th scope="col"></th>
        <th scope="col" style="text-align: center;">Views</th>
    </tr>
    </thead>

    <tbody>

<?php
	kstats_cleanup_searches();

	$query = sprintf( "SELECT terms, engine, viewed, referrer, count FROM `%s` ORDER BY count DESC LIMIT 20", KSTATS_TABLE_SEARCHES );
	$result = $wpdb->get_results ( $query );

	if ( is_null( $result ) || empty( $result ) ): ?>
    <tr><td colspan="4" style="padding: 4em; text-align: center;"><strong>No search terms to report yet.</strong></td></tr>
    <?php
	else: foreach ( $result as $the ): ?>
    <tr><td><a href="<?php echo $the->referrer; ?>"><?php echo urldecode ( $the->terms ); ?></a></td>
        <td><?php echo $the->engine; ?></td>
        <td><a href="<?php bloginfo ( 'url' ); echo ( strpos ( $the->viewed, 'index.php' ) === FALSE ) ? $the->viewed : ''; ?>"><img src="<?php echo KSTATS_PLUGIN_URL; ?>media/page_go.png" width="16" height="16" title="page viewed" /></a></td>
        <td style="text-align: center;"><?php echo $the->count; ?></td>
    </tr> <?php
	endforeach; endif;

?>

    </tbody>
    </table>
    </div>


    <div style="margin: 2em 0; width: 49%; float: right;">
    <h2>Top Referrers</h2>
    <table class="kstatsfat">
    <thead>
    <tr><th scope="col">URL</th>
        <th scope="col"></th>
        <th scope="col" style="text-align: center;">Views</th>
    </tr>
    </thead>

    <tbody>

<?php
	kstats_cleanup_referrers();

	$query = sprintf ( "SELECT viewed, referrer, count FROM `%s` ORDER BY count DESC LIMIT 20", KSTATS_TABLE_REFERRERS );
	$result = $wpdb->get_results ( $query );

	if ( is_null( $result ) || empty( $result ) ): ?>
    <tr><td colspan="3" style="padding: 4em; text-align: center;"><strong>No referrers to report yet.</strong></td></tr>
    <?php
	else: foreach ( $result as $the ): ?>
    <tr><td><a href="<?php echo $the->referrer; ?>"><?php echo kstats_truncate ( $the->referrer, 50 ); ?></a></td>
        <td><a href="<?php bloginfo ( 'url' ); echo ( strpos ( $the->viewed, 'index.php' ) === FALSE ) ? $the->viewed : ''; ?>"><img src="<?php echo KSTATS_PLUGIN_URL; ?>media/page_go.png" width="16" height="16" title="page viewed" /></a></td>
        <td style="text-align: center;"><?php echo $the->count; ?></td>
    </tr> <?php
	endforeach; endif;

?>

		</tbody>
        </table>
		</div>

		<div style="clear: both;"></div>

		<div style="margin: 2em 0; width: 48%; float: left;">
        <h2>Top User Agents</h2>
        <table class="kstatsfat">
        <thead>
        <tr><th scope="col">Name</th>
            <th scope="col" style="text-align: center;">Visits</th>
		</tr>
        </thead>

		<tbody>

<?php
	kstats_cleanup_agents();

	$query = sprintf ( "SELECT user_agent, name, count FROM `%s` ORDER BY count DESC LIMIT 20", KSTATS_TABLE_AGENTS );
	$result = $wpdb->get_results ( $query );

	if ( is_null( $result ) || empty( $result ) ): ?>
    <tr><td colspan="3" style="padding: 4em; text-align: center;"><strong>No user agents to report yet.</strong></td></tr>
    <?php
	else: foreach ( $result as $the ): ?>
    <tr><td><abbr title="<?php echo $the->user_agent; ?>"><?php echo $the->name; ?></abbr></td>
        <td style="text-align: center;"><?php echo $the->count; ?></td>
    </tr> <?php
    endforeach; endif;

?>

    </tbody>
    </table>
    </div>

    <div style="margin: 2em 0; width: 49%; float: right;">
    <h2>Top Crawlers</h2>
    <table class="kstatsfat">
    <thead>
    <tr><th scope="col">Crawler</th>
        <th scope="col">Last Page Visited</th>
        <th scope="col" style="text-align: center;">Visits</th>
    </tr>
    </thead>

    <tbody>

<?php
	kstats_cleanup_spiders();

	$query = sprintf ( "SELECT user_agent, spider, url, count FROM `%s` ORDER BY count DESC LIMIT 20", KSTATS_TABLE_SPIDERS );
	$result = $wpdb->get_results ( $query );

	if ( is_null( $result ) || empty( $result ) ): ?>
    <tr><td colspan="3" style="padding: 4em; text-align: center;"><strong>No crawls to report yet.</strong></td></tr>
    <?php
	else: foreach ( $result as $the ): ?>
    <tr><td><abbr title="<?php echo $the->user_agent; ?>"><?php echo $the->spider; ?></abbr></td>
        <td style="color: #757575;"><abbr title="<?php echo $the->url; ?>"><?php echo kstats_truncate ( $the->url, 30 ); ?></abbr></td>
        <td style="text-align: center;"><?php echo $the->count; ?></td>
    </tr> <?php
    endforeach; endif;

?>

    </tbody>
    </table>
    </div>

    <div style="clear: both;"></div>

<?php          
}


function kstats_display_overview( $totals, $last_month = TRUE ) {
?>
	<table class="kstatsfat">
    <thead>
    <tr><th scope="col"></th>
        <th scope="col">Total</th>
        <?php if ( $last_month ): ?><th scope="col">Last month<span><?php echo gmdate ( 'M, Y', strtotime( '-1 Month', kstats_current_time() ) ); ?></span></th><?php endif; ?>
        <th scope="col">This month<span><?php echo gmdate ( 'M, Y', kstats_current_time() ); ?></span></th>
        <th scope="col">Yesterday<span><?php echo gmdate ( 'd M, Y', kstats_current_time() - 86400 ); ?></span></th>
        <th scope="col">Today<span><?php echo gmdate ( 'd M, Y', kstats_current_time() ); ?></span></th>
	</tr>
    </thead>

	<tbody>
	<tr><td><div class="boxes sc_visitors"></div> Visitors</td>
        <td><?php echo number_format( $totals['visitors']['total'] ); ?></td>
        <?php if ( $last_month ): ?><td><?php echo number_format( $totals['visitors']['last'] ); ?></td><?php endif; ?>
        <td><?php echo number_format( $totals['visitors']['this'] ); ?></td>
        <td><?php echo number_format( $totals['visitors']['yesterday'] ); ?></td>
        <td><?php echo number_format( $totals['visitors']['today'] ); ?></td>
	</tr>

	<tr><td><div class="boxes sc_pageviews"></div> Page Views</td>
        <td><?php echo number_format( $totals['pageviews']['total'] ); ?></td>
        <?php if ( $last_month ): ?><td><?php echo number_format( $totals['pageviews']['last'] ); ?></td><?php endif; ?>
        <td><?php echo number_format( $totals['pageviews']['this'] ); ?></td>
        <td><?php echo number_format( $totals['pageviews']['yesterday'] ); ?></td>
        <td><?php echo number_format( $totals['pageviews']['today'] ); ?></td>
	</tr>

	<tr><td><div class="boxes sc_spiders"></div> Spiders</td>
        <td><?php echo number_format( $totals['spiders']['total'] ); ?></td>
        <?php if ( $last_month ): ?><td><?php echo number_format( $totals['spiders']['last'] ); ?></td><?php endif; ?>
        <td><?php echo number_format( $totals['spiders']['this'] ); ?></td>
        <td><?php echo number_format( $totals['spiders']['yesterday'] ); ?></td>
        <td><?php echo number_format( $totals['spiders']['today'] ); ?></td>
	</tr>

	<tr><td><div class="boxes sc_feeds"></div> Feeds</td>
        <td><?php echo number_format( $totals['feeds']['total'] ); ?></td>
        <?php if ( $last_month ): ?><td><?php echo number_format( $totals['feeds']['last'] ); ?></td><?php endif; ?>
        <td><?php echo number_format( $totals['feeds']['this'] ); ?></td>
        <td><?php echo number_format( $totals['feeds']['yesterday'] ); ?></td>
        <td><?php echo number_format( $totals['feeds']['today'] ); ?></td>
	</tr>

    </tbody>
    </table>
<?php
}

function kstats_display_charts( $charts, $max, $days = 21 ) {
	global $kstatstime;

	if ( ! is_array( $charts ) )
		return;

	$max = ( $max == 0 ) ? 1 : $max;

    $column_width  = ( 95 / $days ) . '%';

	$first_day = $days - 1;

	echo '<table width="100%" class="kstatsfat"><tr>';

    while ( $days-- ) {

		$current = gmdate( 'Y-m-d', kstats_current_time() - 86400 * $days );

		$px_visitors  = round ( $charts[$current]['visitors'] * 100 / $max );
		$px_pageviews = round ( $charts[$current]['pageviews'] * 100 / $max );
		$px_spiders   = round ( $charts[$current]['spiders'] * 100 / $max );
		$px_feeds     = round ( $charts[$current]['feeds'] * 100 / $max );

		$px_white     = 100 - $px_visitors - $px_pageviews - $px_spiders - $px_feeds;

		echo '<td width="' . $column_width . '" valign="bottom" style="border-bottom: none;';
		if ( $kstatstime->start_of_week == gmdate( 'w', kstats_current_time() - 86400 * $days ) && $days != $first_day )
			echo ' border-left: 1px dotted #555; padding-left: 2px; ';
		else
			echo ' padding-left: 1px; ';
		echo '">';

?>
<div style="float: left; height: 100%; width: 100%; font-size: 9px; font-family: Helvetica, Arial, sans-serif; text-align: center;">
<div style="background: #fff; width: 100%; height: <?php echo $px_white; ?>px;"></div>
<div class="sc_visitors"  style="width: 100%; height: <?php echo $px_visitors; ?>px;"  title="<?php echo $charts[$current]['visitors']; ?> visitors"></div>
<div class="sc_pageviews" style="width: 100%; height: <?php echo $px_pageviews; ?>px;" title="<?php echo $charts[$current]['pageviews']; ?> pageviews"></div>
<div class="sc_spiders"   style="width: 100%; height: <?php echo $px_spiders; ?>px;"   title="<?php echo $charts[$current]['spiders']; ?> spiders"></div>
<div class="sc_feeds"     style="width: 100%; height: <?php echo $px_feeds; ?>px;"     title="<?php echo $charts[$current]['feeds']; ?> feeds"></div>
<div style="margin-bottom: 3px; background: #fff; width: 100%; height: 1px;"></div>
<?php

	echo gmdate('d\<\b\r \/\>M', kstats_current_time() - 86400 * $days) . '</div></td>';

	}

	echo '</tr></table>';

}


